@@ -2,9 +2,63 @@
 // Licensed under the MIT license.
 
 #include "ansivt.hpp"
+#include "logger.hpp"
 
 #pragma once
 
+namespace netxs::prompt
+{
+    static constexpr auto  pads = "      "sv;
+    static constexpr auto    os = "  os: "sv;
+    static constexpr auto   tty = " tty: "sv;
+    static constexpr auto   ack = " ack: "sv;
+    static constexpr auto   xml = " xml: "sv;
+    static constexpr auto   vtm = " vtm: "sv;
+    static constexpr auto   cin = "stdin: "sv;
+    static constexpr auto  cout = "stdout: "sv;
+    static constexpr auto win32 = "win32api: "sv;
+
+    #define prompt_list \
+        X(apps) /* */ \
+        X(base) /* */ \
+        X(calc) /* */ \
+        X(desk) /* */ \
+        X(diff) /* */ \
+        X(dtvt) /* */ \
+        X(exec) /* */ \
+        X(foci) /* */ \
+        X(gate) /* */ \
+        X(hall) /* */ \
+        X(hids) /* */ \
+        X(host) /* */ \
+        X(logs) /* */ \
+        X(main) /* */ \
+        X(meet) /* */ \
+        X(open) /* */ \
+        X(page) /* */ \
+        X(para) /* */ \
+        X(path) /* */ \
+        X(pipe) /* */ \
+        X(pool) /* */ \
+        X(rail) /* */ \
+        X(repl) /* */ \
+        X(s11n) /* */ \
+        X(sock) /* */ \
+        X(term) /* */ \
+        X(task) /* */ \
+        X(text) /* */ \
+        X(tile) /* */ \
+        X(user) /* */ \
+        X(vtty) /* */ \
+        X(xipc) /* */ \
+        X(xtty) /* */
+
+    #define X(prompt) static constexpr auto prompt = #prompt": "sv;
+        prompt_list
+    #undef X
+    #undef prompt_list
+}
+
 namespace netxs::directvt
 {
     using namespace netxs::ansi;
@@ -18,36 +72,33 @@ namespace netxs::directvt
         { };
         #pragma pack(push,1)
         static constexpr auto initial = char{ '\xFF' };
-        union marker
+        struct marker
         {
-            struct mask
-            {
-                using sz_t = le_t<size_t>;
-                char mark_FF;
-                sz_t cfgsize;
-                char mark_FE;
-            };
-            static constexpr auto size = sizeof(mask);
-            char data[size];
-            mask pack;
+            using sz_t = le_t<size_t>;
+            char mark_FF;
+            sz_t cfgsize;
+            char mark_FE;
+
             marker()
             { }
-            marker(size_t cfgsize)
+            marker(size_t config_size)
             {
-                pack.mark_FF = initial;
-                pack.cfgsize.set(cfgsize);
-                pack.mark_FE = initial - 1;
+                mark_FF = initial;
+                cfgsize.set(config_size);
+                mark_FE = initial - 1;
             }
-            auto get_sz(size_t& cfgsize)
+            auto get_sz(size_t& config_size)
             {
-                if (pack.mark_FF == initial
-                 && pack.mark_FE == initial - 1)
+                if (mark_FF == initial
+                 && mark_FE == initial - 1)
                 {
-                    cfgsize = pack.cfgsize.get();
+                    config_size = cfgsize.get();
                     return true;
                 }
                 else return faux;
             }
+            auto data() { return reinterpret_cast<char*>(this); }
+            auto size() { return sizeof(*this); }
         };
         #pragma pack(pop)
         struct stream
@@ -109,7 +160,7 @@ namespace netxs::directvt
                 {
                     // Noop.
                 }
-                else log("dtvt: unsupported data type");
+                else log(prompt::dtvt, "Unsupported data type");
             }
             // stream: Replace bytes at specified position.
             template<class T>
@@ -129,13 +180,13 @@ namespace netxs::directvt
                 {
                     if (data.size() < sizeof(sz_t))
                     {
-                        log("dtvt: corrupted view frame");
+                        log(prompt::dtvt, "Corrupted frame header");
                         return view{};
                     }
                     auto size = netxs::letoh(*reinterpret_cast<sz_t const*>(data.data()));
                     if (data.size() < size + sizeof(size))
                     {
-                        log("dtvt: corrupted data");
+                        log(prompt::dtvt, "Corrupted frame data");
                         return view{};
                     }
                     auto crop = view{ data.data() + sizeof(sz_t), (size_t)size };
@@ -164,7 +215,7 @@ namespace netxs::directvt
                     using type = decltype(span{}.count());
                     if (data.size() < sizeof(type))
                     {
-                        log("dtvt: corrupted time data");
+                        log(prompt::dtvt, "Corrupted datetime data");
                         return D{};
                     }
                     auto temp = *reinterpret_cast<type const*>(data.data());
@@ -179,7 +230,7 @@ namespace netxs::directvt
                 {
                     if (data.size() < sizeof(D))
                     {
-                        log("dtvt: corrupted integer data");
+                        log(prompt::dtvt, "Corrupted integer data");
                         return D{};
                     }
                     auto crop = netxs::letoh(*reinterpret_cast<D const*>(data.data()));
@@ -247,7 +298,7 @@ namespace netxs::directvt
             {
                 if (size > data.size())
                 {
-                    log("dtvt: wrong data size");
+                    log(prompt::dtvt, "Wrong data size");
                     return view{};
                 }
                 auto crop = data.substr(0, size);
@@ -265,7 +316,7 @@ namespace netxs::directvt
                     auto step = *reinterpret_cast<sz_t const*>(iter);
                     if (step < sizeof(sz_t))
                     {
-                        log("dtvt: stream corrupted, frame size: ", step);
+                        log(prompt::dtvt, "Stream corrupted", ", frame size: ", step);
                         break;
                     }
                     if (size < step) break;
@@ -355,14 +406,14 @@ namespace netxs::directvt
                 auto shot = input(buff.data(), buff.size());
                 if (shot.size() != buff.size())
                 {
-                    log("dtvt: stream corrupted");
+                    log(prompt::dtvt, "Stream corrupted");
                     return faux;
                 }
                 auto rest = size_t{};
                 rest = *reinterpret_cast<sz_t const*>(buff.data());
                 if (rest < sizeof(sz_t))
                 {
-                    log("dtvt: stream corrupted, frame size: ", rest);
+                    log(prompt::dtvt, "Stream corrupted", ", frame size: ", rest);
                     return faux;
                 }
                 rest -= shot.size();
@@ -373,7 +424,7 @@ namespace netxs::directvt
                     shot = input(head, rest);
                     if (!shot)
                     {
-                        log("dtvt: stream corrupted");
+                        log(prompt::dtvt, "Stream corrupted");
                         return faux;
                     }
                     rest -= shot.size();
@@ -383,7 +434,7 @@ namespace netxs::directvt
                 auto kind = *reinterpret_cast<type const*>(data.data());
                 if (kind != object.kind)
                 {
-                    log("dtvt: object type mismatch");
+                    log(prompt::dtvt, "Object type mismatch");
                     return faux;
                 }
                 data.remove_prefix(sizeof(type));
@@ -464,13 +515,27 @@ namespace netxs::directvt
             void send(T&& sender, Args&&... args)
             {
                 auto lock = freeze();
-                if constexpr (!!sizeof...(args))
+                if constexpr (sizeof...(args))
                 {
                     thing.set(std::forward<Args>(args)...);
                 }
                 thing.template sendby<Discard_empty>(sender);
             }
             // wrapper .
+            template<bool Discard_empty = faux, class T, class ...Args>
+            void send(sptr<T> sender_ptr, Args&&... args)
+            {
+                send<Discard_empty>(*sender_ptr,std::forward<Args>(args)...);
+            }
+            // wrapper .
+            template<class T>
+            auto recv(sptr<T> link)
+            {
+                auto lock = freeze();
+                thing.load([&](auto... args){ return link->recv(args...); });
+                return thing;
+            }
+            // wrapper .
             template<class ...Args>
             auto operator () (Args&&... args)
             {
@@ -520,7 +585,7 @@ namespace netxs::directvt
                         auto size = sz_t{};
                         std::tie(size, item.next) = stream::template take<sz_t, type>(rest);
                         stop = size > rest.size() + head;
-                        if (stop) log("dtvt: corrupted data");
+                        if (stop) log(prompt::dtvt, "Corrupted data");
                         else
                         {
                             crop = rest.substr(0, size - head);
@@ -559,128 +624,125 @@ namespace netxs::directvt
         };
         static constexpr auto _counter_base = __COUNTER__;
         // Definition of plain objects.
-        #define MACROGEN_DEF
+        #define DEFINE_macro
         #include "macrogen.hpp"
-        #define STRUCT(struct_name, struct_members)                               \
-            struct CAT(struct_name, _t) : public stream                           \
-            {                                                                     \
-                static constexpr auto kind = type{ __COUNTER__ - _counter_base }; \
-                SEQ_ATTR(WRAP(struct_members))                                    \
-                CAT(struct_name, _t)()                                            \
-                    : stream{ kind }                                              \
-                { }                                                               \
-                void set(SEQ_SIGN(WRAP(struct_members)) int _tmp = {})            \
-                {                                                                 \
-                    SEQ_INIT(WRAP(struct_members))                                \
-                    stream::reset();                                              \
-                    stream::add(SEQ_NAME(WRAP(struct_members)) noop{});           \
-                }                                                                 \
-                template<class T>                                                 \
-                void set(T&& source)                                              \
-                {                                                                 \
-                    SEQ_TEMP(WRAP(struct_members))                                \
-                    stream::reset();                                              \
-                    stream::add(SEQ_NAME(WRAP(struct_members)) noop{});           \
-                }                                                                 \
-                void get(view& _data)                                             \
-                {                                                                 \
-                    int _tmp;                                                     \
-                    std::tie(SEQ_NAME(WRAP(struct_members)) _tmp) =               \
-                        stream::take<SEQ_TYPE(WRAP(struct_members)) noop>(_data); \
-                }                                                                 \
-                template<class P>                                                 \
-                auto load(P recv)                                                 \
-                {                                                                 \
-                    return stream::read_block(*this, recv);                       \
-                }                                                                 \
-                void wipe()                                                       \
-                {                                                                 \
-                    SEQ_WIPE(WRAP(struct_members))                                \
-                    stream::reset();                                              \
-                }                                                                 \
-                                                                                  \
-                friend std::ostream& operator << (std::ostream& s,                \
-                                                    CAT(struct_name, _t) const& o)\
-                {                                                                 \
-                    s << #struct_name " {";                                       \
-                    SEQ_LOGS(WRAP(struct_members))                                \
-                    s << " }";                                                    \
-                    return s;                                                     \
-                }                                                                 \
-            };                                                                    \
-            using struct_name = wrapper<CAT(struct_name, _t)>;
+        #define STRUCT_macro(struct_name, struct_members)                                   \
+            struct CAT_macro(struct_name, _t) : public stream                               \
+            {                                                                               \
+                static constexpr auto kind = type{ __COUNTER__ - _counter_base };           \
+                SEQ_ATTR_macro(WRAP_macro(struct_members))                                  \
+                CAT_macro(struct_name, _t)()                                                \
+                    : stream{ kind }                                                        \
+                { }                                                                         \
+                void set(SEQ_SIGN_macro(WRAP_macro(struct_members)) int _tmp = {})          \
+                {                                                                           \
+                    SEQ_INIT_macro(WRAP_macro(struct_members))                              \
+                    stream::reset();                                                        \
+                    stream::add(SEQ_NAME_macro(WRAP_macro(struct_members)) noop{});         \
+                }                                                                           \
+                template<class T>                                                           \
+                void set(T&& source)                                                        \
+                {                                                                           \
+                    SEQ_TEMP_macro(WRAP_macro(struct_members))                              \
+                    stream::reset();                                                        \
+                    stream::add(SEQ_NAME_macro(WRAP_macro(struct_members)) noop{});         \
+                }                                                                           \
+                void get(view& _data)                                                       \
+                {                                                                           \
+                    int _tmp;                                                               \
+                    std::tie(SEQ_NAME_macro(WRAP_macro(struct_members)) _tmp) =             \
+                      stream::take<SEQ_TYPE_macro(WRAP_macro(struct_members)) noop>(_data); \
+                }                                                                           \
+                template<class P>                                                           \
+                auto load(P recv)                                                           \
+                {                                                                           \
+                    return stream::read_block(*this, recv);                                 \
+                }                                                                           \
+                void wipe()                                                                 \
+                {                                                                           \
+                    SEQ_WIPE_macro(WRAP_macro(struct_members))                              \
+                    stream::reset();                                                        \
+                }                                                                           \
+                friend std::ostream& operator << (std::ostream& s,                          \
+                                             CAT_macro(struct_name, _t) const& o)           \
+                {                                                                           \
+                    s << #struct_name " {";                                                 \
+                    SEQ_LOGS_macro(WRAP_macro(struct_members))                              \
+                    s << " }";                                                              \
+                    return s;                                                               \
+                }                                                                           \
+            };                                                                              \
+            using struct_name = wrapper<CAT_macro(struct_name, _t)>;
         //todo use C++20 __VA_OPT__ (MSVC not ready yet)
-        #define STRUCT_LITE(struct_name)                                          \
-            struct CAT(struct_name, _t) : public stream                           \
+        #define STRUCT_macro_lite(struct_name)                                    \
+            struct CAT_macro(struct_name, _t) : public stream                     \
             {                                                                     \
                 static constexpr auto kind = type{ __COUNTER__ - _counter_base }; \
-                CAT(struct_name, _t)()                                            \
+                CAT_macro(struct_name, _t)()                                      \
                     : stream{ kind }                                              \
                 { }                                                               \
                 void set() {}                                                     \
                 void get(view& data) {}                                           \
                                                                                   \
                 friend std::ostream& operator << (std::ostream& s,                \
-                                                    CAT(struct_name, _t) const& o)\
+                                             CAT_macro(struct_name, _t) const& o) \
                 {                                                                 \
                     return s << #struct_name " { }";                              \
                 }                                                                 \
             };                                                                    \
-            using struct_name = wrapper<CAT(struct_name, _t)>;
+            using struct_name = wrapper<CAT_macro(struct_name, _t)>;
         //todo unify
         static auto& operator << (std::ostream& s, wchr const& o) { return s << "0x" << utf::to_hex(o); }
         static auto& operator << (std::ostream& s, time const& o) { return s << "0x" << utf::to_hex(o.time_since_epoch().count()); }
         // Output stream.
-        STRUCT(frame_element,     (frag, data))
-        STRUCT(jgc_element,       (ui64, token) (text, cluster))
-        STRUCT(tooltip_element,   (id_t, gear_id) (text, tip_text))
-        STRUCT(mouse_event,       (id_t, gear_id) (hint, cause) (twod, coord) (twod, delta) (ui32, buttons))
-        STRUCT(set_clipboard,     (id_t, gear_id) (twod, clip_prev_size) (text, clipdata) (si32, mimetype))
-        STRUCT(request_clipboard, (id_t, gear_id))
-        STRUCT(set_focus,         (id_t, gear_id) (bool, combine_focus) (bool, force_group_focus))
-        STRUCT(off_focus,         (id_t, gear_id))
-        STRUCT(maximize,          (id_t, gear_id))
-        STRUCT(form_header,       (id_t, window_id) (text, new_header))
-        STRUCT(form_footer,       (id_t, window_id) (text, new_footer))
-        STRUCT(warping,           (id_t, window_id) (dent, warpdata))
-        STRUCT(vt_command,        (text, command))
-        STRUCT_LITE(expose)
-        //todo logs
-        //STRUCT_LITE(request_debug)
+        STRUCT_macro(frame_element,     (frag, data))
+        STRUCT_macro(jgc_element,       (ui64, token) (text, cluster))
+        STRUCT_macro(tooltip_element,   (id_t, gear_id) (text, tip_text) (bool, update))
+        STRUCT_macro(mouse_event,       (id_t, gear_id) (hint, cause) (twod, coord) (twod, delta) (ui32, buttons))
+        STRUCT_macro(keybd_event,       (id_t, gear_id) (ui32, ctlstat) (ui32, winctrl) (ui32, virtcod) (ui32, scancod) (bool, pressed) (ui32, imitate) (text, cluster) (bool, handled))
+        STRUCT_macro(set_clipboard,     (id_t, gear_id) (twod, clip_prev_size) (text, clipdata) (si32, mimetype))
+        STRUCT_macro(request_clipboard, (id_t, gear_id))
+        //STRUCT_macro(focus,             (id_t, gear_id) (bool, state) (bool, focus_combine) (bool, focus_force_group))
+        STRUCT_macro(focus_cut,         (id_t, gear_id))
+        STRUCT_macro(focus_set,         (id_t, gear_id) (si32, solo))
+        STRUCT_macro(fullscreen,        (id_t, gear_id))
+        STRUCT_macro(form_header,       (id_t, window_id) (text, new_header))
+        STRUCT_macro(form_footer,       (id_t, window_id) (text, new_footer))
+        STRUCT_macro(warping,           (id_t, window_id) (dent, warpdata))
+        STRUCT_macro(vt_command,        (text, command))
+        STRUCT_macro(logs,              (ui32, id) (time, guid) (text, data))
+        STRUCT_macro(minimize,          (id_t, gear_id))
+        STRUCT_macro_lite(expose)
         // Input stream.
-        STRUCT(sysfocus,          (id_t, gear_id) (bool, enabled) (bool, combine_focus) (bool, force_group_focus))
-        STRUCT(syskeybd,          (id_t, gear_id) (ui32, ctlstat) (ui32, winctrl) (ui32, virtcod) (ui32, scancod) (bool, pressed) (ui32, imitate) (text, cluster) (wchr, winchar))
-        STRUCT(sysmouse,          (id_t, gear_id)  // sysmouse: Devide id.
-                                  (ui32, enabled)  // sysmouse: Mouse device health status.
-                                  (ui32, ctlstat)  // sysmouse: Keybd modifiers state.
-                                  (ui32, winctrl)  // sysmouse: Windows specific keybd modifier state.
-                                  (ui32, buttons)  // sysmouse: Buttons bit state.
-                                  (bool, doubled)  // sysmouse: Double click.
-                                  (bool, wheeled)  // sysmouse: Vertical scroll wheel.
-                                  (bool, hzwheel)  // sysmouse: Horizontal scroll wheel.
-                                  (si32, wheeldt)  // sysmouse: Scroll delta.
-                                  (twod, coordxy)  // sysmouse: Cursor coordinates.
-                                  (ui32, changed)) // sysmouse: Update stamp.
-        STRUCT(mouse_show,        (bool, mode)) // CCC_SMS/* 26:1p */
-        STRUCT(winsz,             (id_t, gear_id) (twod, winsize))
-        STRUCT(clipdata,          (id_t, gear_id) (text, data) (si32, mimetype))
-        STRUCT(osclipdata,        (id_t, gear_id) (text, data) (si32, mimetype))
-        STRUCT(plain,             (id_t, gear_id) (text, utf8txt))
-        STRUCT(ctrls,             (id_t, gear_id) (ui32, ctlstat))
-        STRUCT(unknown_gc,        (ui64, token))
-        STRUCT(fps,               (si32, frame_rate))
-        STRUCT(bgc,               (rgba, color))
-        STRUCT(fgc,               (rgba, color))
-        STRUCT(slimmenu,          (bool, menusize))
-        STRUCT(startdata,         (text, ip) (text, name) (text, user) (si32, mode) (text, conf))
-        STRUCT(debuglogs,         (time, id) (text, data))
-        //todo logs
-        //STRUCT(debuglogs2,        (text, data))
-        //STRUCT(debugdata,         (text, data))
-        //STRUCT(debugtext,         (text, data))
-        #undef STRUCT
-        #undef STRUCT_LITE
-        #define MACROGEN_UNDEF
+        STRUCT_macro(focusbus,          (id_t, gear_id) (time, guid) (hint, cause))
+        STRUCT_macro(sysfocus,          (id_t, gear_id) (bool, state) (bool, focus_combine) (bool, focus_force_group))
+        STRUCT_macro(syskeybd,          (id_t, gear_id) (ui32, ctlstat) (ui32, winctrl) (ui32, virtcod) (ui32, scancod) (bool, pressed) (ui32, imitate) (text, cluster) (bool, handled))
+        STRUCT_macro(sysmouse,          (id_t, gear_id)  // sysmouse: Devide id.
+                                        (ui32, enabled)  // sysmouse: Mouse device health status.
+                                        (ui32, ctlstat)  // sysmouse: Keybd modifiers state.
+                                        (ui32, winctrl)  // sysmouse: Windows specific keybd modifier state.
+                                        (ui32, buttons)  // sysmouse: Buttons bit state.
+                                        (bool, doubled)  // sysmouse: Double click.
+                                        (bool, wheeled)  // sysmouse: Vertical scroll wheel.
+                                        (bool, hzwheel)  // sysmouse: Horizontal scroll wheel.
+                                        (si32, wheeldt)  // sysmouse: Scroll delta.
+                                        (twod, coordxy)  // sysmouse: Cursor coordinates.
+                                        (ui32, changed)) // sysmouse: Update stamp.
+        STRUCT_macro(mouse_show,        (bool, mode)) // CCC_SMS/* 26:1p */
+        STRUCT_macro(winsz,             (id_t, gear_id) (twod, winsize))
+        STRUCT_macro(clipdata,          (id_t, gear_id) (text, data) (si32, mimetype))
+        STRUCT_macro(osclipdata,        (id_t, gear_id) (text, data) (si32, mimetype))
+        STRUCT_macro(plain,             (id_t, gear_id) (text, utf8txt))
+        STRUCT_macro(ctrls,             (id_t, gear_id) (ui32, ctlstat))
+        STRUCT_macro(unknown_gc,        (ui64, token))
+        STRUCT_macro(fps,               (si32, frame_rate))
+        STRUCT_macro(bgc,               (rgba, color))
+        STRUCT_macro(fgc,               (rgba, color))
+        STRUCT_macro(slimmenu,          (bool, menusize))
+        STRUCT_macro(init,              (text, user) (si32, mode) (text, config))
+        #undef STRUCT_macro
+        #undef STRUCT_macro_lite
+        #define UNDEFINE_macro
         #include "macrogen.hpp"
         // Definition of complex objects.
         class bitmap_t
@@ -709,7 +771,7 @@ namespace netxs::directvt
                 style = 1 << 3,
                 glyph = 1 << 4,
             };
-            void set(id_t winid, twod const& coord, core& cache, bool& abort, sz_t& delta)
+            void set(id_t winid, twod const& coord, core& cache, flag& abort, sz_t& delta)
             {
                 //todo multiple windows
                 stream::reinit(winid, rect{ coord, cache.size() });
@@ -867,7 +929,7 @@ namespace netxs::directvt
                         auto [count] = stream::take<sz_t>(data);
                         if (count > tail - iter)
                         {
-                            log("dtvt: bitmap: corrupted data, subtype: ", what);
+                            log(prompt::dtvt, "bitmap: ", "Corrupted data, subtype: ", what);
                             break;
                         }
                         auto from = iter;
@@ -879,22 +941,22 @@ namespace netxs::directvt
                         auto [offset] = stream::take<sz_t>(data);
                         if (offset >= size)
                         {
-                            log("dtvt: bitmap: corrupted data, subtype: ", what);
+                            log(prompt::dtvt, "bitmap: ", "Corrupted data, subtype: ", what);
                             break;
                         }
                         iter = head + offset;
                     }
                     else // Unknown subtype.
                     {
-                        log("dtvt: bitmap: unknown data, subtype: ", what);
+                        log(prompt::dtvt, "bitmap: ", "Unknown data, subtype: ", what);
                         break;
                     }
                 }
                 image.mark(mark);
-                //log("dtvt: frame len: ", frame_len);
-                //log("dtvt: nop count: ", nop_count);
-                //log("dtvt: rep count: ", rep_count);
-                //log("dtvt: dif count: ", dif_count);
+                //log(prompt::dtvt, "frame len: ", frame_len);
+                //log(prompt::dtvt, "nop count: ", nop_count);
+                //log(prompt::dtvt, "rep count: ", rep_count);
+                //log(prompt::dtvt, "dif count: ", dif_count);
                 //log("----------------------------");
             }
         };
@@ -909,29 +971,33 @@ namespace netxs::directvt
         using request_gc = wrapper<request_gc_t>;
         struct s11n
         {
-            #define OBJECT_LIST \
+            #define object_list \
             /* Output stream                                                      */\
             X(bitmap           ) /* Canvas data.                                  */\
             X(mouse_event      ) /* Mouse events.                                 */\
+            X(keybd_event      ) /* Keybd events.                                 */\
             X(tooltips         ) /* Tooltip list.                                 */\
             X(jgc_list         ) /* List of jumbo GC.                             */\
             X(set_clipboard    ) /* Set main clipboard using following data.      */\
             X(request_clipboard) /* Request main clipboard data.                  */\
-            X(off_focus        ) /* Request to remove focus.                      */\
-            X(set_focus        ) /* Request to set focus.                         */\
-            X(maximize         ) /* Request to maximize/restore                   */\
+            X(focus_cut        ) /* Request to focus cut.                         */\
+            X(focus_set        ) /* Request to focus set.                         */\
+            X(fullscreen       ) /* Request to fullscreen                         */\
             X(form_header      ) /* Set window title.                             */\
             X(form_footer      ) /* Set window footer.                            */\
             X(warping          ) /* Warp resize.                                  */\
+            X(minimize         ) /* Minimize window.                              */\
             X(expose           ) /* Bring the form to the front.                  */\
             X(vt_command       ) /* Parse following vt-sequences in UTF-8 format. */\
             X(frames           ) /* Received frames.                              */\
             X(tooltip_element  ) /* Tooltip text.                                 */\
             X(jgc_element      ) /* jumbo GC: gc.token + gc.view.                 */\
+            X(logs             ) /* Debug logs.                                   */\
             /* Input stream                                                       */\
             X(sysfocus         ) /* System focus state.                           */\
             X(syskeybd         ) /* System keybd device.                          */\
             X(sysmouse         ) /* System mouse device.                          */\
+            X(focusbus         ) /* Focus bus events.                             */\
             X(mouse_show       ) /* Show mouse cursor.                            */\
             X(winsz            ) /* Window resize.                                */\
             X(clipdata         ) /* Clipboard raw data.                           */\
@@ -944,21 +1010,17 @@ namespace netxs::directvt
             X(bgc              ) /* Set background color.                         */\
             X(fgc              ) /* Set foreground color.                         */\
             X(slimmenu         ) /* Set window menu size.                         */\
-            X(startdata        ) /* Startup data.                                 */\
-            X(debuglogs        ) /* Debug logs.                                   */
-            //todo logs
-            //X(debuglogs2       ) /* Debug logs.                                   */
-            //X(debugdata        ) /* Debug data.                                   */
-            //X(debugtext        ) /* Debug forwarding.                             */
-            //X(request_debug    ) /* Request debug output redirection to stdin.    */
+            X(init             ) /* Startup data.                                 */
+            //X(focus            ) /* Request to set focus.                         */\
+
             struct xs
             {
                 #define X(_object) using _object = binary::_object::access;
-                OBJECT_LIST
+                object_list
                 #undef X
             };
             #define X(_object) binary::_object _object;
-            OBJECT_LIST
+            object_list
             #undef X
             std::unordered_map<type, std::function<void(view&)>> exec; // s11n: .
             void sync(view& data)
@@ -971,7 +1033,7 @@ namespace netxs::directvt
                     {
                         iter->second(frame.data);
                     }
-                    else log("s11n: unsupported frame type: ", (int)frame.next, "\n", utf::debase(frame.data));
+                    else log(prompt::s11n, "Unsupported frame type: ", (int)frame.next, "\n", utf::debase(frame.data));
                 }
             }
             s11n() = default;
@@ -981,17 +1043,17 @@ namespace netxs::directvt
                 #define X(_object) \
                     if constexpr (requires(view data) { boss.handle(_object.sync(data)); }) \
                         exec[binary::_object::kind] = [&](auto& data) { boss.handle(_object.sync(data)); };
-                OBJECT_LIST
+                object_list
                 #undef X
                 auto lock = bitmap.freeze();
                 lock.thing.image.link(boss_id);
             }
-            #undef OBJECT_LIST
+            #undef object_list
         };
     }
     namespace ascii
     {
-        template<svga VGAMODE = svga::truecolor>
+        template<svga Mode = svga::truecolor>
         class bitmap
             : public basevt<text>
         {
@@ -1032,7 +1094,7 @@ namespace netxs::directvt
                 return length();
             }
             // ascii::bitmap: .
-            void set(id_t winid, twod const& winxy, core& cache, bool& abort, sz_t& delta)
+            void set(id_t winid, twod const& winxy, core& cache, flag& abort, sz_t& delta)
             {
                 auto coord = dot_00;
                 auto saved = state;
@@ -1045,12 +1107,12 @@ namespace netxs::directvt
                 auto put = [&](cell const& cache)
                 {
                     //todo
-                    cache.scan<VGAMODE>(state, *this);
+                    cache.scan<Mode>(state, *this);
                 };
                 auto dif = [&](cell const& cache, cell const& front)
                 {
                     //todo
-                    return !cache.scan<VGAMODE>(front, state, *this);
+                    return !cache.scan<Mode>(front, state, *this);
                 };
                 auto left_half = [&](cell const& cache)
                 {
